// *******************************************************************
// trigger.cpp
// *******************************************************************


#include <errno.h>
#if !defined(_MSC_VER)
#include <stdexcept>
#endif
#include "uox3.h"
#include "trigger.h"
#include "cScript.h"
#include "fileio.h"
#include "scriptc.h"
#include "ssection.h"

Triggers::Triggers()
{
	ParseScript();
	ParseEnvoke();	// do envoke second, so script builds the script list
}
Triggers::~Triggers()
{
	envokeList.erase( envokeList.begin(), envokeList.end() );
}

void Triggers::Cleanup()
{//this MUST be done BEFORE deleteing Triggers, because ~cScript uses Triggers->Whatever()
	for( std::map< UI16, cScript *>::iterator p = scriptTriggers.begin(); p != scriptTriggers.end(); ++p )
	{
		cScript *pScript = p->second;
		delete pScript;
	}
}

bool Triggers::CheckEnvoke( UI16 itemID ) const
{
	std::map< UI16, UI16 >::const_iterator p = envokeList.find( itemID );
	return ( p != envokeList.end() );
}

UI16 Triggers::GetScriptFromEnvoke( UI16 itemID ) const
{
	std::map< UI16, UI16 >::const_iterator p = envokeList.find( itemID );
	if( p != envokeList.end() )
		return p->second;
	return 0xFFFF;
}

void Triggers::ParseEnvoke( void )
{
	std::ifstream envokefile;
	std::string filename = cwmWorldState->ServerData()->Directory( CSDDP_SCRIPTS ) + "jse_objectassociations.scp";
	if( !FileIO->FileExists( filename ) )
	{
		Console << "Unable to open " << filename << " for parsing" << myendl;
		return;
	}

	Script *fileAssocData	= new Script( filename, NUM_DEFS, false );
	if( fileAssocData != NULL )
	{
		ScriptSection *mSection = fileAssocData->FindEntry( "ENVOKE" );
		if( mSection != NULL )
		{
			UString tag, data;
			for( tag = mSection->First(); !mSection->AtEnd(); tag = mSection->Next() )
			{
				if( !tag.empty() && tag != "\n" && tag != "\r" )
				{
					data			= mSection->GrabData();
					UI16 itemID		= tag.toUShort();
					UI16 scriptID	= data.toUShort();
					cScript *verify	= GetScript( scriptID );
					if( verify != NULL )
						envokeList[itemID] = scriptID;
					else
						Console.Error( 2,"(ENVOKE) itemID 0x%x refers for scriptID 0x%x which does not exist.", itemID, scriptID ); //,myendl;
				}
			}
		}
		else
			Console.Warning( 2, "Envoke section not found, no hard id->script matching being done" );
		delete fileAssocData;
	}
}

//o--------------------------------------------------------------------------o
//|	Function/Class-	
//|	Date					-	09/24/2002
//|	Developer(s)	-	Unknown
//|	Company/Team	-	UOX3 DevTeam
//|	Status				-	
//o--------------------------------------------------------------------------o
//|	Description		-	Parse a JSE Script
//|									
//|	Modification	-	09/24/2002	-	Avtotar - Fixed JSE path usage, to get the 
//|									path to JS file from the uox.ini file
//o--------------------------------------------------------------------------o
//|	Returns				-	NA
//o--------------------------------------------------------------------------o	
void Triggers::ParseScript( void )
{
	std::string scpFileName = cwmWorldState->ServerData()->Directory( CSDDP_SCRIPTS ) + "jse_fileassociations.scp";
	if( !FileIO->FileExists( scpFileName ) )
	{
		Console.Error( 1, "Failed to open %s", scpFileName.c_str() );
		return;
	}

	UI16 trigNum			= 0xFFFF;
	UString basePath		= cwmWorldState->ServerData()->Directory( CSDDP_SCRIPTS );
	Script *fileAssocData	= new Script( scpFileName, NUM_DEFS, false );
	if( fileAssocData != NULL )
	{
		std::map< UI16, cScript *>::iterator p;
		ScriptSection *mSection = fileAssocData->FindEntry( "SCRIPT_LIST" );
		if( mSection != NULL )
		{
			UString tag, data;
			for( tag = mSection->First(); !mSection->AtEnd(); tag = mSection->Next() )
			{
				trigNum	= tag.toUShort();
				data	= mSection->GrabData();
				p		= scriptTriggers.find( trigNum );
				if( p != scriptTriggers.end() )
					scriptTriggers.erase( p );
				UString fullpath = basePath + data;
				if( !FileIO->FileExists( fullpath ) )
				{	// doesn't exist
					Console.Error( 3, "SE mapping of %i to %s failed, file does not exist!", trigNum, data.c_str() );
				}
				else
				{
					try
					{
						cScript *toAdd			= new cScript( fullpath );
						scriptTriggers[trigNum]	= toAdd;
						if( toAdd != NULL )
							RegisterObject( toAdd->Object(), trigNum );
					}
					catch( std::runtime_error &e )
					{
						Console.Error( 2, "Compiling %s caused a construction failure (Details: %s)", fullpath.c_str(), e.what() );
					}
				}
			}
		}
		else
			Console.Warning( 2, "No JS file mappings found" );
		delete fileAssocData;
	}
}


cScript *Triggers::GetScript( UI16 triggerNumber )
{
	std::map< UI16, cScript *>::iterator p = scriptTriggers.find( triggerNumber );
	if( p != scriptTriggers.end() )	//	Script was found
		return p->second;
	else	//	Account was not found
		return NULL;

}

void Triggers::ReloadJS( void )
{
	std::map< UI16, cScript *>::iterator p;
	for( p = scriptTriggers.begin(); p != scriptTriggers.end(); ++p )
		delete p->second;
	scriptTriggers.erase( scriptTriggers.begin(), scriptTriggers.end() );	// erase it all
	ParseScript();
}

void Triggers::PeriodicTriggerCheck( void )
{
	for( UI32 i = 0; i < periodicTriggers.size(); ++i )
	{
		if( cwmWorldState->GetUICurrentTime() >= (UI32)periodicTriggers[i].nextRun )
		{
			if( periodicTriggers[i].trigScript != NULL )
				periodicTriggers[i].trigScript->OnSystemSlice();
		}
	}
}

void Triggers::PushPeriodicTrigger( cScript *trigScript, SI32 nextRun )
{
	periodicTriggers.push_back( periodicTrigger( trigScript, nextRun ) );
}
void Triggers::RegisterObject( JSObject *toReg, UI16 tNum )
{
	scriptToTriggerMapping[toReg] = tNum;
}
void Triggers::UnregisterObject( JSObject *toUnreg )
{
	std::map< JSObject *, UI16 >::iterator toFind = scriptToTriggerMapping.find( toUnreg );
	if( toFind != scriptToTriggerMapping.end() )
		scriptToTriggerMapping.erase( toFind );
}

UI16 Triggers::FindObject( JSObject *toFind )
{
	std::map< JSObject *, UI16 >::iterator tFnd = scriptToTriggerMapping.find( toFind );
	if( tFnd != scriptToTriggerMapping.end() )
		return tFnd->second;
	return 0xFFFF;
}
